/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//*****************************************************************************
//  MSP430FR235x Demo - PIR Sensor Conditioner Circuit: SAC0 + SAC2 + ADC
//
//  Description: Configure SAC0 (first-stage op-amp) and SAC2 (second-stage
//  op-amp) in general-purpose mode in order to connect the appropriate
//  external circuitry. The filtered output of SAC2 (OA2O) is sampled by the
//  ADC window comparator (A8) every ~0.05 seconds using TB1.1 trigger to
//  determine when the signal crosses a threshold, indicating an alert. When
//  the signal drops below LO_THRESHOLD (ADCLO), LED1 turns on. When it exceeds
//  HI_THRESHOLD (ADCHI), LED2 turns on. When it is between LO_THRESHOLD and
//  HI_THRESHOLD (ADCIN), both LED1 and LED2 are off.
//  ACLK = XT1 = 32768 Hz, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                        MSP430FR235x
//                    -------------------
//               /|\ |          P1.1/OA0O|---> Amplified Vo_1
//                |  |          P1.2/OA0-|
//                |  |          P1.3/OA0+|<--- Filtered and biased PIR signal
//                ---|RST                |
//                   |          P3.1/OA2O|---> Amplified Vo_2
//                   |          P3.2/OA2-|
//                   |          P3.3/OA2+|<--- Filtered and biased Vo_1
//                ---|XIN                |
//             32kHz |            P5.0/A8|<--- Filtered Vo_2
//                ---|XOUT               |
//                   |               P1.0|---> LED1 (turns on below ADCLO)
//                   |               P6.6|---> LED2 (turns on above ADCHI)
//                   |                   |
//
//   ***PLEASE REFER TO THE APPLICATION REPORT (SLAA916) FOR A DETAILED
//   ***SCHEMATIC SHOWING HOW TO CONNECT THE PASSIVE COMPONENTS TO THE
//   ***MSP430 PERIPHERALS
//
//   James Evans
//   Texas Instruments Inc.
//   October 2019
//   Built with IAR Embedded Workbench v7.12.2 & Code Composer Studio v9.0.1
//*****************************************************************************
#include <msp430.h>

#define    HI_THRESHOLD    3072          // Assume V(R+) = AVCC = 3.3 V
                                         // 0.75*V(R+) = ~2.475 V
                                         // 0.75*(2^12-bit) = 3072

#define    LO_THRESHOLD    1024          // Assume V(R+) = AVCC = 3.3 V
                                         // 0.25*V(R+) = ~0.825 V
                                         // 0.25*(2^12-bit) = 1024

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;            // Stop watchdog timer
	
	// Configure LED1 (P1.0), LED2 (P6.6)
    P1DIR |= BIT0;                       // Set P1.0 direction as output
    P1OUT &= ~BIT0;                      // Set P1.0 output low
    P6DIR |= BIT6;                       // Set P6.6 direction as output
    P6OUT &= ~BIT6;                      // Set P6.6 output low

    // Configure ADC input
    P5SEL0 |= BIT0;                      // Select A8 (P5.0) function
    P5SEL1 |= BIT0;

    // Configure XT1 oscillator
    P2SEL1 |= BIT6 | BIT7;               // Set P2.6 and P2.7 as crystal pins
    CSCTL4 = SELA__XT1CLK;               // Set ACLK = XT1; MCLK = SMCLK = DCO
    do
    {
        CSCTL7 &= ~(XT1OFFG | DCOFFG);   // Clear XT1 and DCO fault flags
        SFRIFG1 &= ~OFIFG;
    }while (SFRIFG1 & OFIFG);            // Test oscillator fault flag

    // Disable GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;
	
	// Configure ADC window comparator
    ADCCTL0 = ADCSHT_2 | ADCON;            // 16 ADCCLK cycles; Enable ADC core
    ADCCTL1 = ADCSHP | ADCSHS_2 |          // Pulse sample mode; TB1.1 trigger
              ADCCONSEQ_2 |                // Repeat-single-channel mode
              ADCSSEL_0 | ADCDIV_0;        // ADCCLK = MODCLK, divide by 1

    ADCCTL2 = ADCRES_2;                    // 12-bit conversion results
    ADCMCTL0 = ADCINCH_8 | ADCSREF_0;      // A8 ch; V(R+) = AVCC, V(R-) = AVSS
    ADCHI = HI_THRESHOLD;                  // Set comparator high threshold
    ADCLO = LO_THRESHOLD;                  // Set comparator low threshold
    ADCIE |= ADCHIIE | ADCLOIE | ADCINIE;  // Enable ADCHI, ADCLO, ADCIN
                                           // interrupts

    // Configure SAC0
    P1SEL0 |= BIT1 | BIT2 | BIT3;        // Select OA0O (P1.1), OA0- (P1.2)
    P1SEL1 |= BIT1 | BIT2 | BIT3;        // and OA0+ (P1.3) functions

    SAC0OA &= ~(NSEL_0 | PSEL_0);        // Select external sources for SAC0
                                         // negative and positive inputs

    SAC0OA |= NMUXEN | PMUXEN;           // Connect SAC0 negative and positive
                                         // input sources

    SAC0OA |= OAPM;                      // Select low speed and low-power mode
    SAC0OA |= SACEN | OAEN;              // Enable SAC0 and OA in normal mode

    // Configure SAC2
    P3SEL0 |= BIT1 | BIT2 | BIT3;        // Select OA2O (P3.1), OA2- (P3.2)
    P3SEL1 |= BIT1 | BIT2 | BIT3;        // and OA2+ (P3.3) functions

    SAC2OA &= ~(NSEL_0 | PSEL_0);        // Select external sources for SAC2
                                         // negative and positive inputs

    SAC2OA |= NMUXEN | PMUXEN;           // Connect SAC2 negative and positive
                                         // input sources

    SAC2OA |= OAPM;                      // Select low speed and low-power mode
    SAC2OA |= SACEN | OAEN;              // Enable SAC2 and OA in normal mode

    // Configure ADC timer trigger TB1.1
    TB1CCR0 = 1638-1;                    // PWM period; 1638/32768 = ~0.05 s
    TB1CCR1 = 819-1;                     // 50% duty cycle; TB1.1
    TB1CCTL1 = OUTMOD_3;                 // TB1CCR1 set/reset mode
    TB1CTL = TBSSEL_1 | MC_1 | TBCLR;    // ACLK clock source, up mode, clear
    ADCCTL0 |= ADCENC;                   // Enable ADC conversions

    __bis_SR_register(LPM0_bits | GIE);  // Enter LPM0, enable interrupts
    __no_operation();                    // For debugger
}

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV,ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:     // ADCHI; ADC channel > HI_THRESHOLD
            ADCIFG &= ~ADCHIIFG; // Clear interrupt flag
            P1OUT &= ~BIT0;      // Turn off LED1
            P6OUT |= BIT6;       // Turn on LED2
            break;
        case ADCIV_ADCLOIFG:     // ADCLO; ADC channel < LO_THRESHOLD
            ADCIFG &= ~ADCLOIFG; // Clear interrupt flag
            P6OUT &= ~BIT6;      // Turn on LED1
            P1OUT |= BIT0;       // Turn off LED2
            break;
        case ADCIV_ADCINIFG:     // ADCIN;
                                 // LO_THRESHOLD < ADC channel < HI_THRESHOLD

            ADCIFG &= ~ADCINIFG; // Clear interrupt flag
            P1OUT &= ~BIT0;      // Turn off LED1
            P6OUT &= ~BIT6;      // Turn off LED2
            break;
        case ADCIV_ADCIFG:
            break;
        default:
            break;
    }
}
